@@ -1,11 +1,11 @@
 package Language::Prolog::Sugar;
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 use strict;
 use warnings;
 
-use Carp;
+use Carp qw(carp croak);
 use Language::Prolog::Types ':ctors';
 
 
@@ -102,7 +102,11 @@ sub import {
 	    }
 	}
         elsif ($key eq 'auto_functor') {
-            export \&_autoload, $to, 'AUTOLOAD';
+            carp "Language::Prolog::Sugar auto_functor has been obsoleted";
+            export \&_auto_functor, $to, 'AUTOLOAD';
+        }
+        elsif ($key eq 'auto_term') {
+            export \&_auto_term, $to, 'AUTOLOAD';
         }
 	else {
 	    croak "Unknow option '$key'";
@@ -111,7 +115,7 @@ sub import {
 }
 
 our $AUTOLOAD;
-sub _autoload {
+sub _auto_functor {
     my ($pkg, $name) = $AUTOLOAD =~ /(?:(.*)::)?(.*)/;
     $pkg = 'main' unless length $pkg;
     $name =~ /^[A-Z]/
@@ -123,6 +127,22 @@ sub _autoload {
     goto &$AUTOLOAD
 }
 
+sub _auto_term {
+    my ($pkg, $name) = $AUTOLOAD =~ /(?:(.*)::)?(.*)/;
+    $pkg = 'main' unless length $pkg;
+    if ($name =~ /^[A-Z]/) {
+        my $var = prolog_var $name;
+        my $sub = sub () { $var };
+        export $sub, $pkg, $name;
+    }
+    else {
+        export sub { prolog_functor($name, @_) }, $pkg, $name;
+    }
+
+    no strict 'refs';
+    goto &$AUTOLOAD
+}
+
 1;
 __END__
 # Below is stub documentation for your module. You'd better edit it!
@@ -248,13 +268,15 @@ and
 generates the Prolog structure for the example program above.
 
 
-Also, the tag C<auto_functor> can be used to install and AUTOLOAD sub
-on the caller module that would make a functor for every undefined
-subroutine. For instance:
+Also, the tag C<auto_term> can be used to install and AUTOLOAD sub on
+the caller module that would make a functor, term or variable for
+every undefined subroutine. For instance:
 
-  use Language::Prolog::Sugar 'auto_functor';
+  use Language::Prolog::Sugar 'auto_term';
   swi_call(use_module(library(pce)));
+  swi_call(foo(hello, Hello))
 
+The old C<auto_functor> tag has been obsoleted.
 
 =head1 SEE ALSO
 
@@ -263,7 +285,7 @@ L<Language::Prolog::Types>, L<Language::Prolog::Types::Factory>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2002-2006 by Salvador Fandio (sfandino@yahoo.com).
+Copyright 2002-2006 by Salvador FandiE<ntilde>o (sfandino@yahoo.com).
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
